@@ -6,6 +6,7 @@ import gc
 import inspect
 import warnings
 from contextvars import ContextVar, Token
+from functools import wraps
 from typing import Any, Dict, List, Literal, Optional, Type, Union
 
 from curio.meta import from_coroutine
@@ -51,25 +52,19 @@ def reset_session(token: Token[Dict[str, Session | AsyncSession]]) -> None:
 class DBSession:
     def __init__(self, db: SQLAlchemy):
         self.db = db
+        self.child_session_sync = False
+        self.child_session_async = False
 
     def __enter__(self):
         if not isinstance(self.db.sync_session_maker, sessionmaker):
             raise SessionNotInitialisedError
         session = self.db.sync_session_maker(**self.db.sync_session_args)
         session_dict = _session.get()
-        session_dict["sync"][self.db] = session
-        _session.set(session_dict)
-
-        return self.db
-
-    def __enter__(self):
-        if not isinstance(self.db.sync_session_maker, sessionmaker):
-            raise SessionNotInitialisedError
-        session = self.db.sync_session_maker(**self.db.sync_session_args)
-        session_dict = _session.get()
-        session_dict["sync"][self.db] = session
-        _session.set(session_dict)
-
+        if not session_dict["sync"].get(self.db):
+            session_dict["sync"][self.db] = session
+            _session.set(session_dict)
+        else:
+            self.child_session_sync = True
         return self.db
 
     def __exit__(self, exc_type, exc_value, traceback):
@@ -83,10 +78,11 @@ class DBSession:
             except:
                 pass
         try:
-            self.db.sync_session.close()
-            session_dict = _session.get()
-            session_dict["sync"].pop(self.db)
-            _session.set(session_dict)
+            if not self.child_session_sync:
+                self.db.sync_session.close()
+                session_dict = _session.get()
+                session_dict["sync"].pop(self.db)
+                _session.set(session_dict)
         except:
             pass
 
@@ -95,8 +91,11 @@ class DBSession:
             raise SessionNotInitialisedError
         session = self.db.async_session_maker(**self.db.async_session_args)
         session_dict = _session.get()
-        session_dict["async"][self.db] = session
-        _session.set(session_dict)
+        if not session_dict["async"].get(self.db):
+            session_dict["async"][self.db] = session
+            _session.set(session_dict)
+        else:
+            self.child_session_async = True
         return self.db
 
     async def __aexit__(self, exc_type, exc_value, traceback):
@@ -109,10 +108,11 @@ class DBSession:
             except:
                 pass
         try:
-            await self.db.session.close()
-            session_dict = _session.get()
-            session_dict["async"].pop(self.db)
-            _session.set(session_dict)
+            if not self.child_session_async:
+                await self.db.session.close()
+                session_dict = _session.get()
+                session_dict["async"].pop(self.db)
+                _session.set(session_dict)
         except:
             pass
 
@@ -225,6 +225,26 @@ class SQLAlchemy:
         if self.verbose >= 1:
             warnings.warn(message)
 
+    def sync_context(self, func):
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            if not self.initiated:
+                self.init()
+            with self():
+                return func(*args, **kwargs)
+
+        return wrapper
+
+    def async_context(self, func):
+        @wraps(func)
+        async def wrapper(*args, **kwargs):
+            if not self.initiated:
+                self.init()
+            async with self():
+                return await func(*args, **kwargs)
+
+        return wrapper
+
     def _check_optional_components(self):
         exceptions = []
         if not sqlalchemy_asyncio and self.async_:
